# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/system/config/device.proto, blickfeld/system/config/network.proto, blickfeld/system/config/scan_pattern.proto, blickfeld/system/config/time_synchronization.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


class DeviceClass(betterproto.Enum):
    """Enum to distinguish different device classes"""

    CLASS_UNKNOWN = 0
    CUBE = 1
    CUBE_RANGE = 2
    QB2 = 3


@dataclass(eq=False, repr=False)
class Network(betterproto.Message):
    """
    * The "Network" protocol message allows to define and setup the network
    configuration of the Qb2 device which can be further applied by
    "SetConfig()" service API. The message specifies the desired network
    configuration mode: "dhcp", "link-local","static" or their combination, and
    allows custom static configuration of the device. Modes "dhcp" and "static"
    can be applied together. In this scenario, static configuration will be
    used if DHCP configuration fails. Applied configuration is persistent and
    will be available at Qb2 device boot up.
    """

    hostname: str = betterproto.string_field(1)
    """
    * Specifies the persistent system-wide hostname which should be assigned to
    the network device
    """

    dhcp: bool = betterproto.bool_field(2)
    """
    * This setting enables dynamic "DHCP" network configuration mode. The
    "DHCP" mode is used to dynamically get IPv4/v6 network configuration from
    DHCP server in automatic fashion. Can be used standalone or, if required,
    can be combined with "static" configuration settings.
    """

    manual: "NetworkStatic" = betterproto.message_field(3)
    """
    * This setting enables manual "Static" configuration. The "Static" mode
    allows to create custom static configurations for IPv4, IPv6 or their
    combination.
    """

    link_local: bool = betterproto.bool_field(4)
    """
    * (WIP - attention, no fallbacks!) This settings enables automatic "Link-
    local" network configuration mode. In this scenario, device itself selects
    and assigns IP addresses from a particular range. For IPv4: from
    169.254.0.0 through 169.254.255.255. The IPv6 link-local prefix is
    FE80::/10 and is used to create a link-local IPv6 address on an interface
    """

    wireless: "NetworkWireless" = betterproto.message_field(5)
    """
    If the field is set, the device switches from the Ethernet to the WiFi
    interface.
    """


@dataclass(eq=False, repr=False)
class NetworkStatic(betterproto.Message):
    """
    * Static configuration specifies IP settings configuration for IPv4, IPv6
    or their combination. Settings deliver the list of supported IP addresses,
    netmask, gateway, list of DNS servers and DNS search domain.
    """

    ipv4: "NetworkStaticIpv4" = betterproto.message_field(1)
    """Static IPv4 settings"""

    ipv6: "NetworkStaticIpv6" = betterproto.message_field(2)
    """Static IPv6 settings"""

    dns_domain: str = betterproto.string_field(3)
    """
    * The DNS domain field combined together with the hostname define FQDN
    (fully-qualified domain name), that specifies the exact location of Qb2
    device in the tree hierarchy of Domain Name System.
    """


@dataclass(eq=False, repr=False)
class NetworkStaticIpv4(betterproto.Message):
    ip: List[str] = betterproto.string_field(1)
    """
    * List of IPv4 addresses. Each address should be specified in dot-decimal
    format (quad-dotted notation of four decimal integers, ranging from 0 to
    255 each) including netmask prefix as follows "ip_address/prefix". Where
    netmask prefix should be set in CIDR notation (integer ranging from 0 to
    32).
    """

    gateway: str = betterproto.string_field(2)
    """Gateway IPv4 address"""

    dns_server: List[str] = betterproto.string_field(3)
    """List of IPv4 DNS servers"""


@dataclass(eq=False, repr=False)
class NetworkStaticIpv6(betterproto.Message):
    ip: List[str] = betterproto.string_field(1)
    """
    * List of IPv6 addresses. Each address should be specified as eight groups
    of four hexadecimal digits separated by colons and include netmask prefix
    as follows "ip/prefix". Where prefix should be set in CIDR notation
    (integer ranging from 0 to 128). IPv6 shortening rules are also supported.
    """

    gateway: str = betterproto.string_field(2)
    """Gateway IPv6 address"""

    dns_server: List[str] = betterproto.string_field(3)
    """List of IPv6 DNS servers"""


@dataclass(eq=False, repr=False)
class NetworkWireless(betterproto.Message):
    """
    If set, the device will turn of the Ethernet interface and connect to the
    given wireless access point. The Ethernet and WiFi interface cannot work
    simultanously. [NOTE] Always use the validate method of the network service
    to ensure that the given credentials are correct. If the credentials are
    incorrect or the wireless access point is down, the fallback IP address can
    be used via the Ethernet interface to reset the network configuration.
    """

    ssid: str = betterproto.string_field(1)
    """SSID of the wireless network"""

    password: str = betterproto.string_field(2)
    """
    Password of the wireless network [NOTE] If the password is not set and the
    SSID did not change, the existing password is used. This mechanism is
    applied as the password is never returned via the GetConfig API.
    """


@dataclass(eq=False, repr=False)
class ScanPattern(betterproto.Message):
    """
    * The scan pattern defines the movement of the mirrors. The key parameters
    of the pattern are the horizontal and vertical fields of view (FoV) as well
    as the number of scan lines per frame. The frame rate is defined by the
    total number of scan lines and the oscillation frequency of the mirrors
    which is fixed and device-spcific.
    """

    horizontal: "ScanPatternHorizontal" = betterproto.message_field(1)
    vertical: "ScanPatternVertical" = betterproto.message_field(2)
    pulse: "ScanPatternPulse" = betterproto.message_field(3)
    read_only: bool = betterproto.bool_field(4)
    """
    this is an read_only flag and it will be set if this is a default scan
    pattern, which can't be changed or deleted.
    """

    frame_rate: "ScanPatternFrameRate" = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class ScanPatternHorizontal(betterproto.Message):
    """
    * This section defines the movement of the horizontal mirror. The mirror
    moves continuously with its own frequency. Each half-oscillation period of
    the mirror results in one scan line.
    """

    field_of_view: float = betterproto.float_field(1)
    """
    * Unit: [rad] – optical field of view of the horizontal mirror. Note: In
    v2.2* and below, this was defined as a mechanical FoV.
    """


@dataclass(eq=False, repr=False)
class ScanPatternVertical(betterproto.Message):
    """
    * This section defines the movement of the vertical mirror within one
    frame. The mirror moves for from 0 degrees to the configured target FoV
    (up-ramping phase) and back to 0 degrees (down-ramping phase) with its
    eigenfrequency. The number of scanlines for each phase can be configured.
    Due to the dynamics of the mirror, a certain number of scan lines are
    required to reach the target FoV and then to reach 0 degrees again.
    Consequently, if the limit is reached, a lower number of scan lines
    requires a reduced FoV. Thus, a higher number of scan lines allows a larger
    vertical FoV. The vertical FoV is limited by the optical components. Note:
    Due to the time required for one scan line, the rounded even number of scan
    lines directly scales inverse proportionally with the frame rate.
    """

    field_of_view: float = betterproto.float_field(1)
    """
    * Unit: [rad] – FoV in the center of the frame. Due to the eye shape of the
    scan pattern, the vertical FoV decreases the outer boundaries of the
    horizontal mirror.
    """

    scanlines_up: int = betterproto.uint32_field(2)
    """
    * Configures the number of scan lines required for the up-ramping phase.
    During the up-ramping phase, the vertical mirror increases its amplitude
    from 0 degrees to the target FoV. Default: 200
    """

    scanlines_down: int = betterproto.uint32_field(3)
    """
    * Configures the amount of scan lines required for the down-ramping phase.
    During the down-ramping phase, the vertical mirror decreases its amplitude
    from the target FoV to 0 degrees. Default: 30
    """


@dataclass(eq=False, repr=False)
class ScanPatternPulse(betterproto.Message):
    """
    * This section defines the pattern in which the laser pulses and captures
    sample points
    """

    angle_spacing: float = betterproto.float_field(1)
    """
    * Unit: [rad] – This defines the angle within which the horizontal mirror
    moves between two laser pulses. This parameter therefore defines the
    horizontal sampling resolution.
    """


@dataclass(eq=False, repr=False)
class ScanPatternFrameRate(betterproto.Message):
    """This section defines the frame rate of a scan pattern"""

    maximum: float = betterproto.double_field(2)
    """
    Read-only parameter for maximum achievable frame rate of the scan pattern
    """


@dataclass(eq=False, repr=False)
class NamedScanPattern(betterproto.Message):
    """
    Temporary container used for config storage TODO Remove when map type is
    supported.
    """

    name: str = betterproto.string_field(1)
    scan_pattern: "ScanPattern" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class TimeSynchronization(betterproto.Message):
    """
    Conifguration for the system time synchronization The system time is used
    to fill timestamps in generated output data e.g. point cloud frames.
    """

    ntp: "TimeSynchronizationNtp" = betterproto.message_field(1, group="type")
    """Configuration for NTP time synchronization"""


@dataclass(eq=False, repr=False)
class TimeSynchronizationNtp(betterproto.Message):
    """Configuration message for NTP time synchronization"""

    servers: List[str] = betterproto.string_field(1)
    """List of NTP time servers which are to be used"""


@dataclass(eq=False, repr=False)
class Device(betterproto.Message):
    """This message describes a Blickfeld Lidar device"""

    fqdn_or_ip: str = betterproto.string_field(1)
    """Fully-qualified-domain-name / Hostname or IP address"""

    name: str = betterproto.string_field(2)
    """
    Human-readable name of the device Optional: Is filled out by the service if
    not given.
    """

    serial_number: str = betterproto.string_field(3)
    """
    Serial number of the device Optional: Is filled out by the service if not
    given.
    """

    class_: "DeviceClass" = betterproto.enum_field(4)
    """Device class Optional: Is filled out by the service if not given."""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_pipeline/config/background_subtraction.proto, blickfeld/percept_pipeline/config/data_source.proto, blickfeld/percept_pipeline/config/perception.proto, blickfeld/percept_pipeline/config/point_cloud_filter.proto, blickfeld/percept_pipeline/config/zone_algorithm.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import geometry as __base_geometry__


@dataclass(eq=False, repr=False)
class DataSource(betterproto.Message):
    """
    The configuration storing the sources to retrieve the point cloud data for
    further processing.
    """

    cube1_setup: "DataSourceCube1Setup" = betterproto.message_field(
        1, group="data_source_type"
    )
    """Use Cube 1 lidars to retrieve point clouds"""

    qb2_setup: "DataSourceQb2Setup" = betterproto.message_field(
        3, group="data_source_type"
    )
    """Use Qb2 lidars to retrieve point clouds"""


@dataclass(eq=False, repr=False)
class DataSourceCube1(betterproto.Message):
    """Configuration for retrieving point clouds from a Cube 1"""

    fqdn: str = betterproto.string_field(1)
    """The fully qualified domain name of the Cube 1"""

    map_from_lidar: "__base_geometry__.Transform" = betterproto.message_field(2)
    """
    Transformation from the lidar frame to the map frame. If this transform is
    not set, this will be interpreted as identify transformation. Transforming
    all point cloud topics into the map frame results in a globally consistent
    combined point cloud in map frame.
    """


@dataclass(eq=False, repr=False)
class DataSourceQb2(betterproto.Message):
    """Configuration for retrieving point clouds from a Qb2"""

    fqdn: str = betterproto.string_field(1)
    """
    The fully qualified domain name of the Qb2 device. For on-device
    processing/configuration, 'fqdn' can be set to an empty string to request
    the data of the current device.
    """

    map_from_lidar: "__base_geometry__.Transform" = betterproto.message_field(2)
    """
    Transformation from the lidar frame to the map frame. If this transform is
    not set, this will be interpreted as identify transformation. Transforming
    all point cloud topics into the map frame results in a globally consistent
    combined point cloud in map frame.
    """


@dataclass(eq=False, repr=False)
class DataSourceCube1Setup(betterproto.Message):
    """The configuration of a Cube 1 setup of the LiDAR devices"""

    lidars: List["DataSourceCube1"] = betterproto.message_field(1)
    """Cube1 lidars to retrieve the point clouds from"""

    enable_time_sync: bool = betterproto.bool_field(2)
    """
    If true: The same NTP server needs to be configured for each device before
    starting. The lidar clock and and scan-patterns will be continously
    synchronized. Recommended in multi-sensor setups. If false: the lidar
    measurements are timestamps with the receive time by the system. The
    receive time is less accurate if the network is not reliable.
    """


@dataclass(eq=False, repr=False)
class DataSourceTrigger(betterproto.Message):
    """
    Configuration of Trigger Mode. In trigger mode the processing pipeline will
    reduce the frequency at which point clouds and state lists are getting
    computed to reduce the amount of data transmitted between the processing
    device and configured 'lidars'.
    """

    frequency: float = betterproto.float_field(1)
    """
    The frequency at which data is getting requested from the lidars and
    outputs are generated.
    """


@dataclass(eq=False, repr=False)
class DataSourceQb2Setup(betterproto.Message):
    """The configuration of a Qb2 setup of the LiDAR devices"""

    lidars: List["DataSourceQb2"] = betterproto.message_field(1)
    """the qb2 lidars to get the point clouds from"""

    trigger: "DataSourceTrigger" = betterproto.message_field(2)
    """
    If the field is set trigger mode is enabled. If the field is not set, the
    pipeline outputs are computed with the frequency of configured lidars, i.e.
    the scan pattern configured on those.
    """


@dataclass(eq=False, repr=False)
class BackgroundSubtraction(betterproto.Message):
    """
    Algorithms seperating the input point cloud into foreground and background.
    """

    mixture_of_gaussians: "BackgroundSubtractionMixtureOfGaussians" = (
        betterproto.message_field(1, group="background_subtraction_type")
    )
    """Use mixture of gaussians for foreground/background detection"""

    octree: "BackgroundSubtractionOctree" = betterproto.message_field(
        3, group="background_subtraction_type"
    )
    """Use a static octree for foreground/background detection."""


@dataclass(eq=False, repr=False)
class BackgroundSubtractionMixtureOfGaussians(betterproto.Message):
    """
    Dynamic background subtraction using a mixture of gaussians. The algorithm
    automatically slowly updates the background model if the scene changes. The
    update rate is controlled via 'exponential_decay'.
    """

    num_initialization_frames: int = betterproto.uint32_field(1)
    """
    How many frames of the first received frames are used to build the
    background Reasonable default: 10
    """

    exponential_decay: float = betterproto.float_field(2)
    """
    Controls how fast objects switch between foreground and background.
    Exponential decay factor. Reasonable default: 0.005
    """

    min_weight_threshold_for_background: float = betterproto.float_field(3)
    """
    Controls how much noise the background/foreground is expected to have.
    Reasonable default: 0.25
    """


@dataclass(eq=False, repr=False)
class BackgroundSubtractionOctree(betterproto.Message):
    """
    Uses the first received frames to build a static background map of the
    environment. The algorithm requires the scene to only contain the static
    parts of the scene on the start of the processing. The background model is
    not updated automatically afterwards. The ResetBackground grpc method
    allows to update the static map of the environment.
    """

    num_initialization_frames: int = betterproto.uint32_field(1)
    """
    How many frames of the first received frames are used to build the
    background Reasonable default: 10
    """


@dataclass(eq=False, repr=False)
class PointCloudFilter(betterproto.Message):
    """Algorithm for filtering point clouds"""

    radius_outlier: "PointCloudFilterRadiusOutlier" = betterproto.message_field(
        1, group="point_cloud_filter_type"
    )
    """Use radius outlier filter for noise reduction on point clouds"""


@dataclass(eq=False, repr=False)
class PointCloudFilterRadiusOutlier(betterproto.Message):
    """Filter points based on the number of neighbors in a certain radius"""

    min_neighbor_points: int = betterproto.uint32_field(1)
    """Minimum number of neighbors a point has to have to be retained"""

    neighbor_radius: float = betterproto.double_field(2)
    """All points within this radius are considered neighbors of a point"""


@dataclass(eq=False, repr=False)
class Perception(betterproto.Message):
    """
    The configuration of perception algorithms used to run e.g. security zones
    """

    background_subtraction: "BackgroundSubtraction" = betterproto.message_field(1)
    """Extract the foreground point cloud of the scene"""

    foreground_point_cloud_filters: List[
        "PointCloudFilter"
    ] = betterproto.message_field(2)
    """Filter the foreground point cloud"""


@dataclass(eq=False, repr=False)
class ZoneAlgorithm(betterproto.Message):
    """An algorithm that monitors the area within and around a zone."""

    name: str = betterproto.string_field(1)
    """The user readable zone algorithm name"""

    uuid: str = betterproto.string_field(2)
    """The uuid of the zone algorithm (uuid4)"""

    shape: "__base_geometry__.Shape" = betterproto.message_field(3)
    """
    Geometric shape (usually a Box) in which the algorithm detects certain
    behaviors/scenarios
    """

    volume: "ZoneAlgorithmVolume" = betterproto.message_field(9, group="algorithm_type")
    """Run volume monitoring within the zone"""

    security: "ZoneAlgorithmSecurity" = betterproto.message_field(
        7, group="algorithm_type"
    )
    """Run alarm detection within the zone"""

    exclusion: "ZoneAlgorithmExclusion" = betterproto.message_field(
        10, group="algorithm_type"
    )
    """Run exclusion zone"""


@dataclass(eq=False, repr=False)
class ZoneAlgorithmVolume(betterproto.Message):
    """
    Algorithm that measure the volume of a zone The volume of a zone is
    approximated by computing a 'volume map' from the input point clouds. The
    volume map discretize the zone into 'tiles' in the xy-plane with a side-
    length of 'resolution'. Each tile contains the height information about the
    points that fall into this tile. The volume of the volume map can be
    calculated by adding up the volumes of the tiles in the volume map.
    """

    resolution: float = betterproto.float_field(1)
    """
    Grid resolution per tile used to measure the volume. Reasonable default:
    0.1
    """

    empty_volume: float = betterproto.float_field(2)
    """
    The measured volume while area covered by the zone is considered 'empty'.
    The value can be set using TareVolume method while the area of the zone is
    empty. Reasonable default: 0
    """


@dataclass(eq=False, repr=False)
class ZoneAlgorithmSecurity(betterproto.Message):
    """
    Algorithm that triggers an alarm if movement was detected in the zone
    """

    min_points: int = betterproto.uint32_field(1)
    """
    Minimum number of (foreground) points in the security zone to trigger the
    alarm. Reasonable default: 10
    """

    max_points: int = betterproto.uint32_field(2)
    """
    Maximum number of (foreground) points in the security zone that still
    triggers an alarm. More points will disable the alarm. This logic can be
    disabled by setting max_points to '0'. Default: 0
    """


@dataclass(eq=False, repr=False)
class ZoneAlgorithmExclusion(betterproto.Message):
    """Algorithm that excludes points from the input point cloud"""

    pass



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_processing/services/health.proto, blickfeld/percept_processing/services/pipeline.proto, blickfeld/percept_processing/services/point_cloud.proto, blickfeld/percept_processing/services/states.proto, blickfeld/percept_processing/services/volume_map.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    List,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from ...core_processing import data as __core_processing_data__
from ...percept_pipeline import data as __percept_pipeline_data__
from .. import data as _data__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class StatesStreamResponse(betterproto.Message):
    """
    A response for getting a stream of states messages from the running
    pipeline
    """

    states: "_data__.States" = betterproto.message_field(1)
    """The current states with the information detected in the scene."""


@dataclass(eq=False, repr=False)
class VolumeMapStreamRequest(betterproto.Message):
    """A request to receive a stream of state list messages"""

    zone_uuid: str = betterproto.string_field(2)
    """Volume zone uuid"""


@dataclass(eq=False, repr=False)
class VolumeMapStreamResponse(betterproto.Message):
    """
    A response for getting a stream of volume map messages from the running
    pipeline
    """

    volume_map: "_data__.VolumeMap" = betterproto.message_field(1)
    """The current volume map of the requested zone"""


@dataclass(eq=False, repr=False)
class PipelineStartRequest(betterproto.Message):
    """A request to start a headless pipeline."""

    pass


@dataclass(eq=False, repr=False)
class PipelineStartResponse(betterproto.Message):
    """
    A response to PipelineStartRequest that contains the list of available
    streams for the started pipeline.
    """

    data_types: List["_data__.DataType"] = betterproto.enum_field(1)
    """List of available streams"""


@dataclass(eq=False, repr=False)
class PipelineListAvailableDataResponse(betterproto.Message):
    """
    A response containing the list of the data types available for streaming in
    the running pipeline
    """

    data_types: List["_data__.DataType"] = betterproto.enum_field(1)
    """A list of the data types available for streaming"""


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """
    A response for getting a stream of health messages about the pipeline
    """

    health: "_data__.Health" = betterproto.message_field(1)
    """
    The current health message with information about the state of the pipeline
    """


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """A response for getting an health message about the pipeline"""

    health: "_data__.Health" = betterproto.message_field(1)
    """
    The current health message with information about the state of the pipeline
    """


@dataclass(eq=False, repr=False)
class PointCloudStreamRequest(betterproto.Message):
    """
    A request to receive a stream of point cloud messages. Different point
    clouds can be streamed depending on the set fields: - combined point cloud
    (the full point cloud of all the available devices combined) in the map
    coordinate system - the foreground of the combined point cloud in the map
    coordinate system - the complete raw point cloud coming from one device,
    but only in map coordinate system.
    """

    point_cloud_type: "__percept_pipeline_data__.PointCloudType" = (
        betterproto.enum_field(1)
    )
    """The type of point cloud to stream"""

    coordinate_system: "__percept_pipeline_data__.CoordinateSystem" = (
        betterproto.enum_field(2)
    )
    """
    The coordinate system the point cloud should be expressed in (local or
    transformed/map)
    """

    fqdn: str = betterproto.string_field(3, group="point_cloud_source")
    """fqdn of the data source to get the point cloud from"""


@dataclass(eq=False, repr=False)
class PointCloudStreamResponse(betterproto.Message):
    """A response for getting a stream of point cloud messages"""

    point_cloud: "__core_processing_data__.Frame" = betterproto.message_field(1)
    """The required point cloud, based on the fields set in the request"""


class States(betterproto.ServiceStub):
    """
    An RPC service to request a stream of states messages. If a pipeline is
    already running, it will be used to stream the data. If no pipeline is
    running, it will be started first and then the stream will start. NOTE: in
    this second case, the pipeline will also be automatically stopped once
    there are no more clients requesting any stream type from the module.
    """

    async def async_stream(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["StatesStreamResponse"]:
        """A method to get a stream of states messages from the pipeline"""

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.States/Stream",
            request,
            StatesStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["StatesStreamResponse"]:
        """A method to get a stream of states messages from the pipeline"""

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class VolumeMap(betterproto.ServiceStub):
    """An RPC service to request a stream of volume map messages."""

    async def async_stream(
        self,
        *,
        zone_uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["VolumeMapStreamResponse"]:
        """
        A method to get a stream of volume map messages of one volume zone from
        the pipeline
        """

        request = VolumeMapStreamRequest()
        request.zone_uuid = zone_uuid

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.VolumeMap/Stream",
            request,
            VolumeMapStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        *,
        zone_uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["VolumeMapStreamResponse"]:
        """
        A method to get a stream of volume map messages of one volume zone from
        the pipeline
        """

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            zone_uuid=zone_uuid,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class Pipeline(betterproto.ServiceStub):
    """
    An RPC service to start and stop pipelines. Note that currently it is
    impossible to run more than one pipeline at the same time.
    """

    async def async_start(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PipelineStartResponse":
        """
        A method to start a headless pipeline. This means that a pipeline
        started with this method will only be stopped once the Stop method is
        explicitly called
        """

        request = PipelineStartRequest()

        return await self._unary_unary(
            "/blickfeld.percept_processing.services.Pipeline/Start",
            request,
            PipelineStartResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def start(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PipelineStartResponse":
        """
        A method to start a headless pipeline. This means that a pipeline
        started with this method will only be stopped once the Stop method is
        explicitly called
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_start(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_stop(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        A method to stop the running pipeline. This will force-stop the
        pipeline, no matter how the pipeline was created and no matter how many
        clients are streaming from it.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_processing.services.Pipeline/Stop",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def stop(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        A method to stop the running pipeline. This will force-stop the
        pipeline, no matter how the pipeline was created and no matter how many
        clients are streaming from it.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_stop(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_list_available_data(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PipelineListAvailableDataResponse":
        """
        A method to get the list of the data streams available for the running
        pipeline
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_processing.services.Pipeline/ListAvailableData",
            request,
            PipelineListAvailableDataResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list_available_data(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "PipelineListAvailableDataResponse":
        """
        A method to get the list of the data streams available for the running
        pipeline
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list_available_data(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Health(betterproto.ServiceStub):
    """
    An RPC service to request information about the state of the pipeline and
    of the module.
    """

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        A method to get a stream of health messages, containing information
        about the state of the pipeline
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        A method to get a stream of health messages, containing information
        about the state of the pipeline
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """
        A method to get one health message, containing information about the
        state of the pipeline
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_processing.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """
        A method to get one health message, containing information about the
        state of the pipeline
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class PointCloud(betterproto.ServiceStub):
    """
    An RPC service to request a stream of point cloud messages. If a pipeline
    is already running, it will be used to stream the data. If no pipeline is
    running, it will be started first and then the stream will start. NOTE: in
    this second case, the pipeline will also be automatically stopped once the
    this second case, the pipeline will also be automatically stopped once
    there are no more clients requesting any stream type from the module.
    """

    async def async_stream(
        self,
        *,
        point_cloud_type: "__percept_pipeline_data__.PointCloudType" = 0,
        coordinate_system: "__percept_pipeline_data__.CoordinateSystem" = 0,
        fqdn: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["PointCloudStreamResponse"]:
        """
        A method to get a stream of point cloud messages from the pipeline. It
        is possible to stream different point clouds, depending on the fields
        set in the request
        """

        request = PointCloudStreamRequest()
        request.point_cloud_type = point_cloud_type
        request.coordinate_system = coordinate_system
        request.fqdn = fqdn

        async for response in self._unary_stream(
            "/blickfeld.percept_processing.services.PointCloud/Stream",
            request,
            PointCloudStreamResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def stream(
        self,
        *,
        point_cloud_type: "__percept_pipeline_data__.PointCloudType" = 0,
        coordinate_system: "__percept_pipeline_data__.CoordinateSystem" = 0,
        fqdn: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["PointCloudStreamResponse"]:
        """
        A method to get a stream of point cloud messages from the pipeline. It
        is possible to stream different point clouds, depending on the fields
        set in the request
        """

        loop = asyncio.get_event_loop()
        ait = self.async_stream(
            point_cloud_type=point_cloud_type,
            coordinate_system=coordinate_system,
            fqdn=fqdn,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/push/config/authentication.proto, blickfeld/push/config/destination.proto, blickfeld/push/config/payload.proto, blickfeld/push/config/push.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...percept_processing import data as __percept_processing_data__


class PayloadEncoding(betterproto.Enum):
    """define how the messages are serialized"""

    ENCODING_UNSPECIFIED = 0
    """unspecified"""

    ENCODING_PROTOBUF = 1
    """protobuf serialized as binary"""

    ENCODING_JSON = 2
    """json representation of protobuf messages"""

    ENCODING_JSON_FLATTENED = 3
    """flattened json representation of protobuf messages"""


@dataclass(eq=False, repr=False)
class Authentication(betterproto.Message):
    """Authentication definition"""

    user: str = betterproto.string_field(1)
    """the user name used for authentication"""

    password: str = betterproto.string_field(2)
    """the password used for authentication"""

    server_certificate: bytes = betterproto.bytes_field(3)
    """the server certificate (will implicitly use SSL)"""

    client_certificate: bytes = betterproto.bytes_field(4)
    """client certificate"""

    client_key: bytes = betterproto.bytes_field(5)
    """client key"""


@dataclass(eq=False, repr=False)
class Destination(betterproto.Message):
    """Push destination definition"""

    mqtt: "DestinationMqtt" = betterproto.message_field(1, group="destination_type")
    """mqtt server destination"""


@dataclass(eq=False, repr=False)
class DestinationMqtt(betterproto.Message):
    """Mqtt Endpoint definition"""

    server_fqdn: str = betterproto.string_field(1)
    """mqtt server location"""

    server_port: int = betterproto.uint32_field(2)
    """
    mqtt server port will be set to 1883 if it is not set (server_port==0)
    """

    authentication: "Authentication" = betterproto.message_field(3)
    """mqtt server authentication settings"""

    client_identifier: str = betterproto.string_field(4)
    """mqtt client identifier"""

    topic: str = betterproto.string_field(5)
    """
    mqtt topic - if empty the
    'blickfeld/{data_type}/{encoding_type}/{config_name}' pattern is used as a
    default
    """


@dataclass(eq=False, repr=False)
class Payload(betterproto.Message):
    """defines what type of data stream to use as payload"""

    encoding: "PayloadEncoding" = betterproto.enum_field(1)
    """payload encoding"""

    data_type: "__percept_processing_data__.DataType" = betterproto.enum_field(3)
    """payload data type to stream"""


@dataclass(eq=False, repr=False)
class Push(betterproto.Message):
    """defines a combination of stream and destination"""

    name: str = betterproto.string_field(1)
    """name of push"""

    payload: "Payload" = betterproto.message_field(2)
    """stream payload"""

    destination: "Destination" = betterproto.message_field(3)
    """push destination"""

    enabled: bool = betterproto.bool_field(4)
    """push enabled"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/detector/services/health.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import data as _data__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """Response message for get health request."""

    health: "_data__.Health" = betterproto.message_field(1)
    """Health state"""


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """Response message for watch health request."""

    health: "_data__.Health" = betterproto.message_field(1)
    """Health state"""


class Health(betterproto.ServiceStub):
    """
    The health service provides methods to monitor the instant operational
    status of the detector module
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health status of the detector module"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.detector.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health status of the detector module"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        Can be used to attach to the health monitoring status information of
        the detector module
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.detector.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        Can be used to attach to the health monitoring status information of
        the detector module
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_processing/data/data_type.proto, blickfeld/percept_processing/data/health.proto, blickfeld/percept_processing/data/state.proto, blickfeld/percept_processing/data/states.proto, blickfeld/percept_processing/data/volume_map.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    Dict,
    List,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import numpy as np

from ...base import (
    data as __base_data__,
    geometry as __base_geometry__,
)
from ...percept_pipeline import data as __percept_pipeline_data__


class DataType(betterproto.Enum):
    """The type of data that the stream contains"""

    DATA_TYPE_UNSPECIFIED = 0
    """The zero value should not be used"""

    DATA_TYPE_HEALTH = 1
    """
    Health data type: this contains information about the health of the
    pipeline and of the module
    """

    DATA_TYPE_POINT_CLOUD = 2
    """Point cloud data type: a complete point cloud message is streamed"""

    DATA_TYPE_STATES = 5
    """
    States data type: a map of states generated by the system as configured in
    the pipeline
    """

    DATA_TYPE_VOLUME_MAP = 6
    """
    Volume map data type: a volume map generated by one volume zone configured
    in the pipeline
    """


@dataclass(eq=False, repr=False)
class State(betterproto.Message):
    """
    A state transmits high-level information about the scene or setup. The
    information it contains depends on the configuration. States will get
    calculated on every frame of measurement.
    """

    generator_name: str = betterproto.string_field(8)
    """The name of the entity generating this state"""

    security: "StateSecurity" = betterproto.message_field(5, group="state_type")
    """The security state contains intrusion information of a zone"""

    volume: "StateVolume" = betterproto.message_field(7, group="state_type")
    """Current volume measured in the selected zone"""


@dataclass(eq=False, repr=False)
class StateSecurity(betterproto.Message):
    """A message containing an alarm state of a zone."""

    alarm: bool = betterproto.bool_field(1)
    """If there is an alarm or not in the zone"""

    num_detected_points: int = betterproto.uint32_field(2)
    """The current number of lidar points detected in the zone"""


@dataclass(eq=False, repr=False)
class StateVolume(betterproto.Message):
    """
    A message containing the volume state of a zone. This is based on one
    volume zone.
    """

    volume: float = betterproto.float_field(1)
    """The current numerical volume of the zone."""

    coverage: float = betterproto.float_field(5)
    """The current zone volume surface coverage"""


@dataclass(eq=False, repr=False)
class States(betterproto.Message):
    """
    A message that contains a map of states. A state is an information
    generated by the system based on the select zone type or device. For
    example, a state can contain information about the volume or the intrusion
    in the zone.
    """

    timestamp: int = betterproto.uint64_field(1)
    """The timestamp of the measurement(s) used to compute the states."""

    states: Dict[str, "State"] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Map of states where the key is the generator UUID of the state."""


@dataclass(eq=False, repr=False)
class VolumeMap(betterproto.Message):
    """A message containing the volume map of one volume zone."""

    timestamp: int = betterproto.uint64_field(1)
    """Timestamp of the lidar frame in which the volume map was calculated"""

    generator_uuid: str = betterproto.string_field(2)
    """The uuid of the entity generating this state"""

    generator_name: str = betterproto.string_field(3)
    """The name of the entity generating this state"""

    shape: "__base_geometry__.Shape" = betterproto.message_field(4)
    """
    The shape of the volume map, defining the pose and the dimension of the
    volume map in world frame. The shape is similar to the shape of the input
    zone with the following difference: - shape.pose.position:
    (x=zone.pose.position.x, y=zone.pose.position.y, z=0) -
    shape.pose.rotation: (roll=0, pitch=0, yaw=zone.pose.rotation.yaw) -
    shape.box.dimension: (x=zone.box.dimension.x, y=zone.box.dimension.y,z=0)
    """

    tiles: "VolumeMapTiles" = betterproto.message_field(5)
    """
    Volume map 'tiles' as defined in Tiles, providing information for each tile
    in the volume map.
    """

    tile_size: float = betterproto.float_field(6)
    """
    The tile size of the volume map (dimension of one tile in x and y
    direction)
    """

    volume: float = betterproto.float_field(7)
    """The current volume of the zone"""


@dataclass(eq=False, repr=False)
class VolumeMapTiles(betterproto.Message):
    """Tiles of the volume map"""

    length: int = betterproto.uint32_field(1)
    """Number of tiles"""

    index: np.ndarray = betterproto.bytes_field(2, numpy_dtype="(2,)<i2")
    """
    Volume Map Tile Index X (Int16) and Y (Int16) (can be negative and
    positive) The indices of Volume Tiles are expressed w.r.t the Volume Map
    coordinate system. The tile with index (0,0) will contain the points from
    [0, tile_size), [0, tile_size). That means that (0,0,0) of the Volume Map
    frame does not coincide with the center of the tile with index (0,0) but
    with the crossing of the 4 tiles [(0,0), (0, -1,), (-1,-1), (-1,0)]. Type:
    (Int16, Int16)
    """

    height: np.ndarray = betterproto.bytes_field(3, numpy_dtype="<f4")
    """Height per tile, ordered ascending Type: Float32"""

    std_deviation: np.ndarray = betterproto.bytes_field(4, numpy_dtype="<f4")
    """Standard Deviation per tile Type: Float32"""

    number_points: np.ndarray = betterproto.bytes_field(5, numpy_dtype="<u2")
    """Number Points per tile Type: UInt16"""


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """
    A health message that contains information about the pipeline status and
    the module itself.
    """

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module."""

    state_reason: str = betterproto.string_field(2)
    """Reason for the given state. It is not set if state is STATE_OK."""

    pipeline_state: "__percept_pipeline_data__.State" = betterproto.enum_field(4)
    """
    Current state of the pipeline. It indicates if the pipeline is stable,
    unstable or broken, based on the frequency of the data that is processed
    and published.
    """

    clients: List["HealthClient"] = betterproto.message_field(5)
    """Health information of all connected clients"""


@dataclass(eq=False, repr=False)
class HealthClient(betterproto.Message):
    """Health information of connected clients"""

    uuid: str = betterproto.string_field(1)
    """Identifier of the client"""

    peer_uri: str = betterproto.string_field(2)
    """The client peer uri"""

    connection_timestamp: int = betterproto.uint64_field(3)
    """The timestamp of the moment the client connected to processing"""

    lost_frames: int = betterproto.uint64_field(4)
    """The number of total lost frames since the start of the stream"""

    point_cloud: "HealthClientPointCloud" = betterproto.message_field(
        5, group="stream_type"
    )
    """The client receives point cloud stream data"""

    states: "HealthClientStates" = betterproto.message_field(6, group="stream_type")
    """The client receives states stream data"""

    volume_map: "HealthClientVolumeMap" = betterproto.message_field(
        7, group="stream_type"
    )
    """The client receives volume map stream data"""


@dataclass(eq=False, repr=False)
class HealthClientPointCloud(betterproto.Message):
    """Point Cloud specific information"""

    point_cloud_type: "__percept_pipeline_data__.PointCloudType" = (
        betterproto.enum_field(1)
    )
    """The point cloud type the client is receiving"""


@dataclass(eq=False, repr=False)
class HealthClientStates(betterproto.Message):
    """States specific information"""

    pass


@dataclass(eq=False, repr=False)
class HealthClientVolumeMap(betterproto.Message):
    """Volume Map specific information"""

    pass



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/diagnostics/data/health.proto, blickfeld/diagnostics/data/log.proto, blickfeld/diagnostics/data/self_test_report.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__
from .. import config as _config__


class LogEntryLevel(betterproto.Enum):
    """The log level represents the priority of the log."""

    LEVEL_UNSPECIFIED = 0
    """No level is set"""

    LEVEL_ERROR = 1
    """
    An error condition was detected Includes the systemd levels: EMERGENCY,
    ALERT, CRITICAL, ERROR
    """

    LEVEL_WARNING = 2
    """
    The message indicates that an error will occur if no action is taken
    Includes the systemd levels: NOTICE, WARNING
    """

    LEVEL_INFO = 3
    """
    Normal operational messages that require no action Includes the systemd
    levels: INFORMATIONAL
    """


class SelfTestReportResultFlag(betterproto.Enum):
    """Result flags which indicates if test was sucessful."""

    FLAG_UNSPECIFIED = 0
    """Unknown result."""

    FLAG_SUCCESS = 1
    """The test was successful."""

    FLAG_WARNING = 2
    """
    The test suceeded but it raised some warnings. Please report if this
    happens during production and the root-cause is unknown.
    """

    FLAG_FAILURE = 3
    """The test failed. The tested module is most probably not functional."""


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Qb2 modules"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """
    High-level aggregated health state of the device (if one of the modules
    reports FAILED -> aggregated state is also FAILED)
    """

    state_reason: str = betterproto.string_field(2)
    """
    Reason for given state (user-friendly). Is not set if state is STATE_OK.
    Aggregated string from modules state reasons
    """

    module: "HealthModule" = betterproto.message_field(3)
    """Qb2 modules"""


@dataclass(eq=False, repr=False)
class HealthModule(betterproto.Message):
    """Qb2 modules"""

    laser: "HealthModuleHealth" = betterproto.message_field(1)
    """health of laser module"""

    detector: "HealthModuleHealth" = betterproto.message_field(2)
    """health of detector module"""

    beam_deflection_control_vertical: "HealthModuleHealth" = betterproto.message_field(
        3
    )
    """health of beam deflection control module (vertical)"""

    beam_deflection_control_horizontal: "HealthModuleHealth" = (
        betterproto.message_field(4)
    )
    """health of beam deflection control module (horizontal)"""

    system: "HealthModuleHealth" = betterproto.message_field(6)
    """health of the Blickfeld system module"""

    core_processing: "HealthModuleHealth" = betterproto.message_field(7)
    """health of the Blickfeld core processing module"""

    percept_processing: "HealthModuleHealth" = betterproto.message_field(8)
    """health of percept processing"""

    percept_pipeline: "HealthModuleHealth" = betterproto.message_field(9)
    """health of percept pipeline"""

    push: "HealthModuleHealth" = betterproto.message_field(10)
    """Health of the Blickfeld Push module"""


@dataclass(eq=False, repr=False)
class HealthModuleHealth(betterproto.Message):
    """Message representing health of a Qb2 module"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level health state of the module"""

    state_reason: str = betterproto.string_field(2)
    """
    Reason for given state (user-friendly). Is not set if state is STATE_OK.
    """


@dataclass(eq=False, repr=False)
class Log(betterproto.Message):
    """
    This message aggregates the data structures related to the log service.
    """

    pass


@dataclass(eq=False, repr=False)
class LogEntry(betterproto.Message):
    """The log entry contains the message and metadata of a log."""

    boot_uuid: str = betterproto.string_field(1)
    """UUID of the boot during which the log was written."""

    cursor: "LogEntryCursor" = betterproto.message_field(2)
    """Cursor used to index log entries and to navigate through the logs."""

    timestamp: int = betterproto.uint64_field(3)
    """
    The wallclock time at the point in time the entry was received by the
    journal, in nanoseconds since the epoch UTC.
    """

    module: "_config__.LogFilterSoftwareModule" = betterproto.enum_field(4)
    """The module which has written the log."""

    level: "LogEntryLevel" = betterproto.enum_field(5)
    """The log level of the log."""

    message: str = betterproto.string_field(6)
    """The message which has been logged."""


@dataclass(eq=False, repr=False)
class LogEntryCursor(betterproto.Message):
    """The cursor uniquely identifies a log entry in the logs."""

    sequence_number_id: str = betterproto.string_field(1)
    """
    The sequence number ID is used to reference a set of log entries generated
    by the same systemd-journald instance.
    """

    sequence_number: int = betterproto.uint64_field(2)
    """
    The sequence number is assigned to a log entry by the systemd-journald
    instance when they are written to disk. Every instance numbers their
    entries sequentially, starting from 1 for the first entry written after
    subsystem initialization.
    """


@dataclass(eq=False, repr=False)
class SelfTestReport(betterproto.Message):
    """Report which contains the results of the selected self tests."""

    result_flag: "SelfTestReportResultFlag" = betterproto.enum_field(1)
    """
    Aggregated result flag which is FLAG_SUCCESS if all tests passed without
    warnings
    """

    results: List["SelfTestReportResult"] = betterproto.message_field(2)
    """List of test results"""


@dataclass(eq=False, repr=False)
class SelfTestReportResult(betterproto.Message):
    """Result of test."""

    test: "_config__.SelfTest" = betterproto.enum_field(1)
    """Test case"""

    identifier: str = betterproto.string_field(5)
    """Unique identifier of test case"""

    flag: "SelfTestReportResultFlag" = betterproto.enum_field(2)
    """Flag which states test result"""

    reason: str = betterproto.string_field(3)
    """
    Human-readable reason for test warning or failure. Is not set if test was
    successful.
    """

    duration: float = betterproto.float_field(4)
    """Duration of test run"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/core_processing/config/demo_recording.proto, blickfeld/core_processing/config/health.proto, blickfeld/core_processing/config/point_cloud.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from .. import data as _data__


class DemoRecording(betterproto.Enum):
    """
    Demo recording which can be replayed instead of the real sensor data. This
    is also used for virtual devices which have no physical sensing unit.
    WARNING: The recordings are still preliminary. The demo test data concept
    will be refactored soon.
    """

    DEMO_RECORDING_UNSPECIFIED = 0
    """No demo recoding is specified"""

    DEMO_RECORDING_HIGH_RESOLUTION = 1
    """High resolution recording in the office with 200 + 30 scanlines"""

    DEMO_RECORDING_HIGH_FRAME_RATE = 2
    """High resolution recording in the office with 30 + 30 scanlines"""


class PointCloudFilterSorting(betterproto.Enum):
    """
    Configure sorting of return candidates. The sorting is always descending
    thus the higher the value is, the lower is the return id.
    """

    SORTING_UNSPECIFIED = 0
    """No sorting specified."""

    SORTING_INTENSITY = 1
    """Sort returns by intensity (integral of return pulse)."""

    SORTING_REFLECTIVITY = 2
    """
    Sort returns by reflectivity (range-compensated intensity). This can
    improve the SNR as closer points are getting a lower weight.
    """

    SORTING_RANGE = 3
    """
    Sort returns by range. This can improve the SNR for outdoor applications
    with rain and snow.
    """


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Configuration parameters for the health message"""

    point_rate_lower_limit: int = betterproto.uint32_field(1)
    """
    Set the point rate lower limit. If the point rate is lower than this limit,
    the device is considered to be covered.
    """

    point_cloud_alignment_deviation_norm_upper_limit: float = betterproto.float_field(2)
    """The limit to detect the alignment has changed."""

    point_cloud_reference_alignment: "_data__.Acceleration" = betterproto.message_field(
        3
    )
    """Set the point cloud reference alignment."""


@dataclass(eq=False, repr=False)
class PointCloud(betterproto.Message):
    """Configuration parameters for the point cloud"""

    pass


@dataclass(eq=False, repr=False)
class PointCloudFilter(betterproto.Message):
    """Filter configuration which can be applied on point cloud frames."""

    maximum_returns_per_point: int = betterproto.uint32_field(1)
    """
    Set maximum number of returns per point. By default, secondary returns are
    disabled. Set this, e.g. to 2, to enable secondary returns.
    """

    sorting: "PointCloudFilterSorting" = betterproto.enum_field(3)
    """
    Select sorting method for return candidates. The default sorting is by
    intensity.
    """

    minimum_reflectivity: int = betterproto.uint32_field(4)
    """
    Set lower threshold for reflectivity. This can be tuned to improve the SNR
    when low-reflectivity targets have a lower priority.
    """



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/flow/services/credentials.proto, blickfeld/flow/services/flow.proto, blickfeld/flow/services/settings.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import config as _config__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class FlowGetRequest(betterproto.Message):
    """Flow get request"""

    id: str = betterproto.string_field(1)
    """Flow id"""

    as_node_red_json: bool = betterproto.bool_field(2)
    """
    If true, the request returns the NodeRed JSON format which can be directly
    imported.
    """


@dataclass(eq=False, repr=False)
class FlowGetResponse(betterproto.Message):
    """Flow get response"""

    flow: "_config__.Flow" = betterproto.message_field(1, group="kind")
    """Blickfeld Flow config with well-defined high-level attributes"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(2, group="kind")
    """NodeRed JSON format which can be directly imported"""


@dataclass(eq=False, repr=False)
class FlowListResponse(betterproto.Message):
    """Flow list response"""

    ids: List[str] = betterproto.string_field(1)
    """List of Blickfeld Flow identifiers."""


@dataclass(eq=False, repr=False)
class FlowStoreRequest(betterproto.Message):
    """Flow store request"""

    id: str = betterproto.string_field(1)
    """
    ID of the flow NOTE: For node_red_json, this has to match the ID in the
    NodeRED JSON format.
    """

    flow: "_config__.Flow" = betterproto.message_field(2, group="kind")
    """Blickfeld Flow config with well-defined high-level attributes"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(3, group="kind")
    """
    NodeRed JSON format which can be directly exported from NodeRED Admin UI
    """


@dataclass(eq=False, repr=False)
class FlowDeleteRequest(betterproto.Message):
    """Flow delete request"""

    id: str = betterproto.string_field(1)
    """Id of Blickfeld Flow"""


@dataclass(eq=False, repr=False)
class FlowStoreGlobalNodesRequest(betterproto.Message):
    """Flow store global nodes request"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(1)
    """NodeRed JSON format of all global configuration nodes"""


@dataclass(eq=False, repr=False)
class FlowGetGlobalNodesResponse(betterproto.Message):
    """Flow get global nodes response"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(1)
    """NodeRed JSON format of all global configuration nodes"""


@dataclass(eq=False, repr=False)
class SettingsSetRequest(betterproto.Message):
    """Settings set request"""

    settings: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """
    Settings which should be stored Generic JSON format which is defined by
    NodeRed.
    """


@dataclass(eq=False, repr=False)
class SettingsGetResponse(betterproto.Message):
    """Settings get request"""

    settings: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """Current settings Generic JSON format which is defined by NodeRed."""


@dataclass(eq=False, repr=False)
class CredentialsSetRequest(betterproto.Message):
    """Credentials set request"""

    credentials: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """
    Credentials which should be stored Generic JSON format which is defined by
    NodeRed.
    """


@dataclass(eq=False, repr=False)
class CredentialsGetResponse(betterproto.Message):
    """Credentails get response"""

    credentials: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """Current credentials Generic JSON format which is defined by NodeRed."""


class Flow(betterproto.ServiceStub):
    """
    Service to store and control Blickfeld flows. It is used to implement the
    custom storage API: https://nodered.org/docs/api/storage/
    """

    async def async_get(
        self,
        *,
        id: str = "",
        as_node_red_json: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "FlowGetResponse":
        """Get a flow by id"""

        request = FlowGetRequest()
        request.id = id
        request.as_node_red_json = as_node_red_json

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/Get",
            request,
            FlowGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        *,
        id: str = "",
        as_node_red_json: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "FlowGetResponse":
        """Get a flow by id"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                id=id,
                as_node_red_json=as_node_red_json,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowListResponse":
        """Return list of flow ids"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/List",
            request,
            FlowListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowListResponse":
        """Return list of flow ids"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_store(
        self,
        *,
        id: str = "",
        flow: "_config__.Flow" = None,
        node_red_json: "_config__.NodeRedJson" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store flow"""

        request = FlowStoreRequest()
        request.id = id
        if flow is not None:
            request.flow = flow
        if node_red_json is not None:
            request.node_red_json = node_red_json

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/Store",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def store(
        self,
        *,
        id: str = "",
        flow: "_config__.Flow" = None,
        node_red_json: "_config__.NodeRedJson" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store flow"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_store(
                id=id,
                flow=flow,
                node_red_json=node_red_json,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_delete(
        self,
        *,
        id: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete stored flow"""

        request = FlowDeleteRequest()
        request.id = id

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/Delete",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def delete(
        self,
        *,
        id: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete stored flow"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_delete(
                id=id,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_store_global_nodes(
        self,
        *,
        node_red_json: "_config__.NodeRedJson" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store global nodes"""

        request = FlowStoreGlobalNodesRequest()
        if node_red_json is not None:
            request.node_red_json = node_red_json

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/StoreGlobalNodes",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def store_global_nodes(
        self,
        *,
        node_red_json: "_config__.NodeRedJson" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store global nodes"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_store_global_nodes(
                node_red_json=node_red_json,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get_global_nodes(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowGetGlobalNodesResponse":
        """Get global nodes"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/GetGlobalNodes",
            request,
            FlowGetGlobalNodesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_global_nodes(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowGetGlobalNodesResponse":
        """Get global nodes"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_global_nodes(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Settings(betterproto.ServiceStub):
    """
    Configuration service to store the NodeRed Settings. It is used to
    implement the custom storage API: https://nodered.org/docs/api/storage/
    """

    async def async_set(
        self,
        *,
        settings: "betterproto_lib_google_protobuf.Struct" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set settings Used by https://nodered.org/docs/api/storage/methods/#stor
        agesavesettingssettings
        """

        request = SettingsSetRequest()
        if settings is not None:
            request.settings = settings

        return await self._unary_unary(
            "/blickfeld.flow.services.Settings/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        settings: "betterproto_lib_google_protobuf.Struct" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set settings Used by https://nodered.org/docs/api/storage/methods/#stor
        agesavesettingssettings
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                settings=settings,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "SettingsGetResponse":
        """
        Get settings Used by
        https://nodered.org/docs/api/storage/methods/#storagegetsettings
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Settings/Get",
            request,
            SettingsGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "SettingsGetResponse":
        """
        Get settings Used by
        https://nodered.org/docs/api/storage/methods/#storagegetsettings
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Credentials(betterproto.ServiceStub):
    """
    Service to store the NodeRed credentials. It is used to implement the
    custom storage API: https://nodered.org/docs/api/storage/
    """

    async def async_set(
        self,
        *,
        credentials: "betterproto_lib_google_protobuf.Struct" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set credentials Used by https://nodered.org/docs/api/storage/methods/#s
        toragesavecredentialscredentials
        """

        request = CredentialsSetRequest()
        if credentials is not None:
            request.credentials = credentials

        return await self._unary_unary(
            "/blickfeld.flow.services.Credentials/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        credentials: "betterproto_lib_google_protobuf.Struct" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set credentials Used by https://nodered.org/docs/api/storage/methods/#s
        toragesavecredentialscredentials
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                credentials=credentials,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "CredentialsGetResponse":
        """
        Get credentials Used by
        https://nodered.org/docs/api/storage/methods/#storagegetcredentials
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Credentials/Get",
            request,
            CredentialsGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "CredentialsGetResponse":
        """
        Get credentials Used by
        https://nodered.org/docs/api/storage/methods/#storagegetcredentials
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/push/data/health.proto, blickfeld/push/data/status.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import Dict

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


class StatusConnection(betterproto.Enum):
    """Connection status of the push"""

    CONNECTION_UNSPECIFIED = 0
    """unspecified"""

    CONNECTION_DISABLED = 1
    """push configuration is disabled"""

    CONNECTION_DISCONNECTED = 2
    """disconnected from destination"""

    CONNECTION_CONNECTING = 3
    """connecting to destination"""

    CONNECTION_CONNECTED = 4
    """connected to destination"""

    CONNECTION_PUSHING = 5
    """connected to input and destination"""


@dataclass(eq=False, repr=False)
class Status(betterproto.Message):
    """state of a push"""

    sent_messages: int = betterproto.uint32_field(1)
    """message statistics [default = 0]"""

    connection: "StatusConnection" = betterproto.enum_field(2)
    """flag to indicate if it is working or not"""

    data_rate: float = betterproto.float_field(3)
    """
    Average data rate The resulting value is averaged over the last 5 messages
    sent, peak or current values might be different.
    """

    last_message: datetime = betterproto.message_field(4)
    """Timestamp when last message was sent"""

    state: "__base_data__.HealthState" = betterproto.enum_field(5)
    """State of push connection."""

    state_reason: str = betterproto.string_field(6)
    """
    Reason for the given state. Needs to be set if state is not STATE_OK.
    """


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """A health message that contains information about the push status"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module."""

    state_reason: str = betterproto.string_field(2)
    """
    Reason for the given state. Needs to be set if state is not STATE_OK.
    """

    status: Dict[str, "Status"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """map of push status"""



from . import core_processing
from . import laser
from . import percept_pipeline
from . import diagnostics
from . import percept_toolkit
from . import push
from . import detector
from . import base
from . import system
from . import beam_deflection_control
from . import percept_processing
from . import hardware
from . import flow
from .base.grpc.channel import Channel



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/hardware/config/identification.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


@dataclass(eq=False, repr=False)
class Identification(betterproto.Message):
    """
    Identification message of the Qb2 device. It aggregates all hardware
    components.
    """

    serial_number: str = betterproto.string_field(1)
    """
    This is the top-level product identification. The serial number will be
    used as label and API identifier for the device.
    """



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/percept_toolkit/services/geometry.proto, blickfeld/percept_toolkit/services/health.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterable,
    AsyncIterator,
    Dict,
    Iterable,
    Iterator,
    List,
    Optional,
    Union,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from ...base import geometry as __base_geometry__
from ...core_processing import data as __core_processing_data__
from .. import data as _data__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class GeometryComputeGroundAlignmentRequest(betterproto.Message):
    """
    Request for estimating alignment to ground/gravity based on imu-data and
    optional point cloud frame.
    """

    accelerometer_data: "__core_processing_data__.Acceleration" = (
        betterproto.message_field(1)
    )
    """
    accelerometer data is assumed to be expressed in the lidar coordinate frame
    """

    lidar_frame: "__core_processing_data__.Frame" = betterproto.message_field(2)
    """
    if a lidar frame is provided, it is used to estimate the z-coordinate of
    the translation as well.
    """


@dataclass(eq=False, repr=False)
class GeometryComputeGroundAlignmentResponse(betterproto.Message):
    """The estimated alignment to the ground/gravity."""

    alignment: "__base_geometry__.Transform" = betterproto.message_field(1)
    """
    Estimated alignment of the sensor to the ground-plane/gravity. If no lidar
    frame is provided only roll, pitch of the rotation are estimated. Otherwise
    the z-component of the translation is additionally aligned with a flat
    ground. The transformation transforms the lidar point clouds such that the
    points are aligned with the ground.
    """


@dataclass(eq=False, repr=False)
class GeometryComputeRegistrationRefinementRequest(betterproto.Message):
    """
    The request for transformation refinement based on 3d point clouds and an
    initial point cloud transformations.
    """

    frame: "__core_processing_data__.Frame" = betterproto.message_field(1)
    """lidar frame used for aligning the 3d point clouds"""

    transform: "__base_geometry__.Transform" = betterproto.message_field(2)
    """
    Transform: the transformation transforming a point in from the local lidar
    coordinate system into a 'combined' reference system the 'combined'
    reference system can also be a lidar.
    """


@dataclass(eq=False, repr=False)
class GeometryComputeRegistrationRefinementResponse(betterproto.Message):
    """The refined lidar transformations."""

    transformations: List["__base_geometry__.Transform"] = betterproto.message_field(1)
    """
    Refined transformations. The array sequence is equal to the sequence in
    which the requests were received. The first element in the array
    corresponds to the lidar frame received in the first client stream message.
    """


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """Health Stream response of the toolkit module"""

    health: "_data__.Health" = betterproto.message_field(1)
    """The current health state of the module"""


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """Health Get response of the toolkit module"""

    health: "_data__.Health" = betterproto.message_field(1)
    """The current health state of the module"""


class Geometry(betterproto.ServiceStub):
    """Service offering on-demand geometric processing tasks."""

    async def async_compute_ground_alignment(
        self,
        *,
        accelerometer_data: "__core_processing_data__.Acceleration" = None,
        lidar_frame: "__core_processing_data__.Frame" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GeometryComputeGroundAlignmentResponse":
        """
        Computes the transformation aligning the lidar point cloud to the
        ground plane/gravity.
        """

        request = GeometryComputeGroundAlignmentRequest()
        if accelerometer_data is not None:
            request.accelerometer_data = accelerometer_data
        if lidar_frame is not None:
            request.lidar_frame = lidar_frame

        return await self._unary_unary(
            "/blickfeld.percept_toolkit.services.Geometry/ComputeGroundAlignment",
            request,
            GeometryComputeGroundAlignmentResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def compute_ground_alignment(
        self,
        *,
        accelerometer_data: "__core_processing_data__.Acceleration" = None,
        lidar_frame: "__core_processing_data__.Frame" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GeometryComputeGroundAlignmentResponse":
        """
        Computes the transformation aligning the lidar point cloud to the
        ground plane/gravity.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_compute_ground_alignment(
                accelerometer_data=accelerometer_data,
                lidar_frame=lidar_frame,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_compute_registration_refinement(
        self,
        request_iterator: Union[
            AsyncIterable["GeometryComputeRegistrationRefinementRequest"],
            Iterable["GeometryComputeRegistrationRefinementRequest"],
        ],
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "GeometryComputeRegistrationRefinementResponse":
        """
        Refines the point cloud registration based on provided initial
        transformations. If the method succeeds, the refined transformation is
        returned. Otherwise an exception will be thrown.
        """

        return await self._stream_unary(
            "/blickfeld.percept_toolkit.services.Geometry/ComputeRegistrationRefinement",
            request_iterator,
            GeometryComputeRegistrationRefinementRequest,
            GeometryComputeRegistrationRefinementResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def compute_registration_refinement(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "GeometryComputeRegistrationRefinementResponse":
        """
        Refines the point cloud registration based on provided initial
        transformations. If the method succeeds, the refined transformation is
        returned. Otherwise an exception will be thrown.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_compute_registration_refinement(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Health(betterproto.ServiceStub):
    """An gRPC service to request the health of toolkit"""

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        Streams health messages in regular intervals containing the state of
        toolkit
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.percept_toolkit.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        Streams health messages in regular intervals containing the state of
        toolkit
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Return health messages containing the state of toolkit"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.percept_toolkit.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Return health messages containing the state of toolkit"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/diagnostics/options.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


@dataclass(eq=False, repr=False)
class SoftwareModuleEnumValueOption(betterproto.Message):
    """Options for the software modules"""

    systemd_unit_name: str = betterproto.string_field(1)
    """Name of the systemd unit corresponding to this software module"""
